/***************************************************************
 * Name:      wxextprocess.h
 * Purpose:
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2009-05-20
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   wxWindows licence
 **************************************************************/

#ifndef WXEXTPROCESS_H
#define WXEXTPROCESS_H

// Include additional wxWidgets headers.
#include <wx/process.h>


//////////////////////////////////////////////////////////
// Class wxExtProcess
//
/// \brief An extended wxProcess based class.
///
/// It extends wxProcess in a way that it is possible to
/// kill an external process by saving the process id
class wxExtProcess : public wxProcess
{
  public:
    /// \brief Standard constructor copied from base class.
    ///
    /// \param poParent         a pointer to the event handler parent.
    /// \param nId              id of an event
    wxExtProcess( wxEvtHandler * poParent, int nId= -1 )
      : wxProcess( poParent, nId ), m_nProcessId( 0 )
    {
    }

    /// Virtual destructor.
    virtual ~wxExtProcess()
    {
    }

    /// \brief Execute the exernal process and save the process id.
    ///
    /// \param strCommand       command to execute
    /// \param nSync            flag to indicate wether to execute the
    ///                         process synchronously or asynchronously.
    virtual long Execute( const wxString& strCommand, int nSync= wxEXEC_ASYNC )
    {
        m_nProcessId= ::wxExecute( strCommand, nSync, this );
        return m_nProcessId;
    }

    /// \brief Set the process id of the external process.
    ///
    /// \param nProcessId   process id of the external process
    virtual void SetProcessId( long nProcessId )
    {
        m_nProcessId= nProcessId;
    }

    /// \brief Get the process id of the external process.
    virtual long GetProcessId()
    {
        return m_nProcessId;
    }

    /// \brief "Kill" a running process.
    ///
    /// \param nSignal      signal (see description of wxProcess)
    /// \param nFlags       flags (see description of wxProcess)
    virtual wxKillError Kill( wxSignal nSignal= wxSIGNONE, int nFlags= wxKILL_NOCHILDREN )
    {
        return wxProcess::Kill( GetProcessId(), nSignal, nFlags );
    }

  protected:
    // Member variables
    long m_nProcessId;      ///< Holds the process id of the process.
};


//////////////////////////////////////////////////////////
// Class wxExtProcessDelTempFile
//
/// \brief A slightly extended wxExtProcess class that
///        removes a temporary file in its destructor.
///
/// This class is usefull if you want to be sure that a
/// possibly existing temporary file should be removed
/// after process termination even it the process was
/// killed by Kill().
class wxExtProcessDelTempFile : public wxExtProcess
{
  public:
    /// \brief Standard constructor.
    ///
    /// \param poParent         a pointer to the event handler parent
    /// \param nId              id of an event
    /// \param strTempFileName  the file name to remove after termination
    wxExtProcessDelTempFile( wxEvtHandler *poParent= NULL, int nId = -1,
                             const wxString &strTempFileName= wxEmptyString )
      : wxExtProcess( poParent, nId )
    {
        m_strTempFileName= strTempFileName;
    }

    /// Virtual destructor. Removes the temporary file.
    virtual ~wxExtProcessDelTempFile()
    {
        if( !m_strTempFileName.IsEmpty() && ::wxFileExists( m_strTempFileName ) )
        {
            // Remove the temporary file.
            ::wxRemoveFile( m_strTempFileName );
        }
    }

  protected:
    wxString m_strTempFileName;     ///< Holds the name of the temporary file to be removed in the destructor.
};

#endif // WXEXTPROCESS_H
