--[[**********************************************************
    * Name:      wxiscan.lua
    * Purpose:   Demo for implementing Lua addons for wxIScan.
    * Author:    Daniel Nell (daniel.nell@nellresearch.de)
    * Created:   2009-11-13
    * Copyright: Daniel Nell (www.nellresearch.de)
    * License:   GPL
    **********************************************************
--]]



-------------------------------------------------------------
-- Redefine output function(s).
--
-- Replacement of the standard Lua print procedure that
-- sends the output to the LogMessage function which wraps
-- ::wxLogMessage. So output goes to the standard log target
-- of the wxWidgets bases main program.
--
function print( ... )
    local args= {...}       -- Table for the function parameters.
    local text= ""          -- Accumulates the text for the output.

    -- Iterate on the function parameters and concatenate
    for index, value in ipairs( args ) do
        text= text .. tostring( value ) .. "\t"
    end

    -- Output via LogMessage (wxLuaScript::LogMessage).
    LogMessage( text )
end



-------------------------------------------------------------
-- Usefull functions calling menu handlers on the C++ side.
--
-- Scan an image.
--
function ScanImage()
    return SendMenuEvent( ID_FILESCANIMAGE )
end


-- Print an image.
--
-- NOTE: This opens the printing dialog before printing.
--
function PrintImage()
    return SendMenuEvent( ID_FILEPRINT )
end


-- Do OCR and read out the text.
--
-- NOTE: The read out may be done asynchronously if
--       bAsync is set to 'true'.
--
function DocumentReadText( bAsync )
    -- Handle default parameter(s).
    if bAsync == nil then
        bAsync= true
    end

    if bAsync then
        return SendMenuEvent( ID_DOCUMENTREADTEXT )
    else
        return SendMenuEvent( ID_DOCUMENTREADTEXTSYNC )
    end
end


-- Turn a multiple image document one page back.
--
function DocumentPreviousPage()
    return SendMenuEvent( ID_DOCUMENTPREVIOUSPAGE )
end


-- Turn a multiple image document one page forward.
--
function DocumentNextPage()
    return SendMenuEvent( ID_DOCUMENTNEXTPAGE )
end


-- Stop an asynchronous running sub process.
--
function CmdStop()
    return SendMenuEvent( ID_CMDSTOP )
end



-------------------------------------------------------------
-- Main chunks
-- (referred by the "Script" section in the config file).
--
-- Scan an image and send it to the printer.
--
function CarbonCopy()
    return ScanImage() and PrintImage()
end


-- Scan an image and read out its text (using OCR).
--
function ScanAndRead()
    return ScanImage() and CmdStop() and DocumentReadText()
end


-- Turn pages of a multiple image document (TIFF or PDF)
-- and read out page by page.
--
function TurnPagesAndRead()
    if CmdStop() and DocumentReadText( false ) then
        while DocumentNextPage() and CmdStop() and DocumentReadText( false ) do
        end
    end
end


-- Do OCR via cuneiform, export it as an RTF document,
-- and open OpenOffice.org as RTF editor.
--
function Ocr2Ooo( Language )
    -- Handle default parameter(s).
    if Language == nil then
        Language= "de"
    end

    -- Create a temporary file(name).
    local TempFileName= os.tmpname()

    -- Create temporary filename(s) from basename.
    local TempFileNameText= TempFileName .. ".rtf"

    -- OCR commands for various languages.
    local OcrCmd=
    {
        ["de"]= "cuneiform -l ger -f rtf -o " .. TempFileNameText .. " " .. TempFileName;
        ["en"]= "cuneiform -l eng -f rtf -o " .. TempFileNameText .. " " .. TempFileName
    }

    -- Convert the image to a monochrome image.
    Image2Mono()

    -- Save the image to the given name (as tiff file).
    SaveImage( TempFileName, "image/tiff" )

    -- Execute the OCR software and create a rich text file.
    os.execute( OcrCmd[Language] )

    -- Open openoffice.org for editing.
    os.execute( "soffice " .. TempFileNameText )

    -- Remove temporary files.
    os.remove( TempFileName )
    os.remove( TempFileNameText )
end


-- Do OCR via cuneiform, export it as hOCR document,
-- turn it into a seekable PDF document using hocr2pdf
-- from ExcactImage and open it using evince.
--
function Ocr2SeekablePdf( Language )
    -- Handle default parameter(s).
    if Language == nil then
        Language= "de"
    end

    -- Create a temporary file(name).
    local TempFileName= os.tmpname()

    -- Create temporary filename(s) from basename.
    local TempFileNameText= TempFileName .. ".hocr"
    local TempFileNamePdf= TempFileName .. ".pdf"

    -- OCR commands for various languages.
    local OcrCmd=
    {
        ["de"]= "cuneiform -l ger -f hocr -o " .. TempFileNameText .. " " .. TempFileName;
        ["en"]= "cuneiform -l eng -f hocr -o " .. TempFileNameText .. " " .. TempFileName
    }

    -- Convert the image to a monochrome image.
    Image2Mono()

    -- Save the image to the given name (as tiff file).
    SaveImage( TempFileName, "image/tiff" )

    -- Execute the OCR software and create a rich text file.
    os.execute( OcrCmd[Language] )

    -- Combine tiff image and hocr html-text into a seekable PDF file.
    os.execute( "hocr2pdf -i " .. TempFileName .. " -o " .. TempFileNamePdf .. " < " .. TempFileNameText )

    -- Open the PDF file.
    os.execute( "evince " .. TempFileNamePdf )

    -- Remove temporary files.
    os.remove( TempFileName )
    os.remove( TempFileNameText )
    os.remove( TempFileNamePdf )

    -- Remove temporary image directory created by cuneiform.
    os.execute( "rm -r " .. TempFileName .. "_files" )
end


-- Do OCR via cuneiform using the given language and
-- read it out using espeak, mbrola, aplay and the
-- given language.
--
function ReadOut( Language )
    -- Handle default parameter(s).
    if Language == nil then
        Language= "de"
    end

    -- Create a temporary file(name).
    local TempFileName= os.tmpname()

    -- Create temporary filename(s) from basename.
    local TempFileNameText= TempFileName .. ".txt"

    -- OCR commands for various languages.
    local OcrCmd=
    {
        ["de"]= "cuneiform -l ger -f text -o " .. TempFileNameText .. " " .. TempFileName;
        ["en"]= "cuneiform -l eng -f text -o " .. TempFileNameText .. " " .. TempFileName
    }

    -- Read out commands for various languages.
    local ReadOutCmd=
    {
        ["de"]= "sh -c 'espeak -p 130 -v mb-de6 -f " .. TempFileNameText
                .. " | mbrola -e /usr/share/mbrola/de6/de6 - - | aplay -r22050 -fS16'";
        ["en"]= "sh -c 'espeak -s 100 -p 50 -v mb-en1 -f " .. TempFileNameText
                .. " | mbrola -e /usr/share/mbrola/en1/en1 - - | aplay -r22050 -fS16'"
    }

    -- Convert the image to a monochrome image.
    Image2Mono()

    -- Save the image to the given name (as TIFF file).
    SaveImage( TempFileName, "image/tiff" )

    -- Execute the OCR software and create a plain text file.
    Execute( OcrCmd[Language] )

    -- Read out the text file.
    Execute( ReadOutCmd[Language], true, TempFileNameText )

    -- Remove temporary files.
    os.remove( TempFileName )
end


-- 'Unpaper' (deskew) the program's image using unpaper
-- and the give options.
--
function Unpaper( Options )
    -- Handle default parameter(s).
    if Options == nil then
        Options= ""
    end

    -- Create temporary file(name)s.
    local TempFileName= os.tmpname()
    local TempFileNameOut= os.tmpname()

    -- Save input image.
    SaveImage( TempFileName, "image/pnm" )

    -- Execute unpaper program.
    local Cmd= "unpaper " .. Options .. " --overwrite " .. TempFileName .. " " .. TempFileNameOut
    os.execute( Cmd )

    -- Load output image
    OpenImage( TempFileNameOut )

    -- Remove temporary files.
    os.remove( TempFileName )
    os.remove( TempFileNameOut )
end


-- Test function for Lua bindings.
--
function Test()
    print( "LUA: " .. _VERSION )
    print( "LUA: Current date and time is " .. os.date( "%x %X", os.time() ) .. '.' )
    print( "LUA: Your home directory is '" .. os.getenv( "HOME" ) .. "'." )
    print( "LUA: package.cpath == '" .. package.cpath .. "'" )

--    EnhenceColours( -10, 20 )
--    Image2Mono()
    RotateImage( 9 / 360 * ( 2 * math.pi ) )
--    EnhenceOpticalContrast()
--    ResetFileName()
end
